﻿using IgApp.Models;
using Infragistics.Web.Mvc;
using Microsoft.AspNetCore.Mvc;

using System.Text;
using System.Text.Json;
using System.Text.RegularExpressions;

namespace IgApp.Controllers
{
    public class HomeController : Controller
    {
        [GridDataSourceAction]
        public ActionResult Index()
        {
            return View();
        }

        public JsonResult GetData()
        {
            IQueryable<Employee> data = GetEmployees().AsQueryable();
            int totalCount = data.Count();

            if (Request.Query.Keys.Any(x => x.Contains("filter")))
            {
                data = ApplyFiltering(Request.Query, data);
            }

            JsonResult result = new JsonResult(new { }, new JsonSerializerOptions { PropertyNamingPolicy = null });
            result.Value = new { Records = data, TotalRecordsCount = totalCount };
            return result;
        }

        private IQueryable<Employee> ApplyFiltering(IQueryCollection queryString, IQueryable<Employee> data)
        {
            List<FilterExpression> exprs = GetFilteringExpressions(queryString);
            StringBuilder builder = new StringBuilder();
            int count = 0;

            for (int i = 0; i < exprs.Count; i++)
            {
                if (count != 0 && count <= exprs.Count - 1)
                {
                    builder.Append(exprs[i].Logic.ToLower() == "AND".ToLower() ? " AND " : " OR ");
                }
                count++;

                string condition = exprs[i].Condition;
                string expr = exprs[i].Expr;
                string colKey = exprs[i].Key;

                switch (condition.ToLower())
                {
                    case "startswith":
                        builder.Append(colKey + ".StartsWith(\"" + expr + "\")");
                        break;
                    case "endswith":
                        builder.Append(colKey + ".EndsWith(\"" + expr + "\")");
                        break;
                    case "contains":
                        builder.Append(colKey + ".Contains(\"" + expr + "\")");
                        break;
                    case "doesnotcontain":
                        builder.Append("! " + colKey + ".Contains(\"" + expr + "\")");
                        break;
                    case "equals":
                        if (colKey != "ID" && colKey != "Age")
                        {
                            //col type is string
                            builder.Append(colKey + " == \"" + expr + "\"");
                        }
                        else
                        {
                            //col type is number
                            builder.Append(colKey + " == " + expr);
                        }
                        break;
                    case "doesnotequal":
                        if (colKey != "ID" && colKey != "Age")
                        {
                            //col type is string
                            builder.Append(colKey + " != \"" + expr + "\"");
                        }
                        else
                        {
                            //col type is number
                            builder.Append(colKey + " != " + expr);
                        }
                        break;
                    case "greaterthan":
                        builder.Append(colKey + " > " + expr);
                        break;
                    case "lessthan":
                        builder.Append(colKey + " < " + expr);
                        break;
                    case "greaterthanorequalto":
                        builder.Append(colKey + " >= " + expr);
                        break;
                    case "lessthanorequalto":
                        builder.Append(colKey + " <= " + expr);
                        break;
                    default:
                        break;
                }
            }
            if (builder.Length > 0)
            {
                data = data.Where(builder.ToString(), new object[0]);
            }

            return data;
        }

        private List<FilterExpression> GetFilteringExpressions(IQueryCollection queryString)
        {
            List<FilterExpression> expressions = new List<FilterExpression>();

            foreach (string key in queryString.Keys)
            {
                if (!string.IsNullOrEmpty(key) && key.StartsWith("filter("))
                {
                    string val = key.Substring(key.IndexOf("(")).Replace("(", "").Replace(")", "");
                    string columnKey = val.Substring(0, val.IndexOf(":"));
                    string logic = "AND";

                    Regex filtersRegex = new Regex(@"[a-z]+\(.*?\)", RegexOptions.IgnoreCase);
                    MatchCollection m = filtersRegex.Matches(queryString[key]);
                    string[] filters = new string[m.Count];

                    int i = 0;
                    foreach (Match capture in m)
                    {
                        filters[i] = capture.Value;
                        i++;
                    }

                    for (i = 0; i < filters.Length; i++)
                    {
                        FilterExpression e = new FilterExpression();
                        e.Logic = logic;
                        e.Key = columnKey;
                        e.Condition = filters[i].Substring(0, filters[i].IndexOf("("));
                        if ((filters[i].StartsWith("contains()") || filters[i].StartsWith("equals()") || filters[i].StartsWith("startsWith()")
                                || filters[i].StartsWith("endsWith()") || filters[i].StartsWith("doesNotContain()") || filters[i].StartsWith("doesNotEqual()")
                            )
                            && (e.Condition != "null" && e.Condition != "notNull" && e.Condition != "empty" && e.Condition != "notEmpty")
                            && (e.Condition != "today" && e.Condition != "yesterday" && e.Condition != "thisMonth" && e.Condition != "nextMonth"
                            && e.Condition != "lastMonth" && e.Condition != "thisYear" && e.Condition != "nextYear" && e.Condition != "lastYear")
                            )
                        {
                            continue;
                        }
                        else
                        {
                            e.Expr = filters[i].Substring(filters[i].IndexOf("(")).Replace("(", "").Replace(")", "");
                        }
                        expressions.Add(e);
                    }
                }
            }

            return expressions;
        }

        private IEnumerable<Employee> GetEmployees()
        {
            return new List<Employee>
            {
                new Employee { ID = 1, FirstName = "John", LastName = "Doe", Country = "USA", Age = 25 },
                new Employee { ID = 2, FirstName = "Jane", LastName = "Doe", Country = "Canada", Age = 26 },
                new Employee { ID = 3, FirstName = "Bob", LastName = "Smith", Country = "USA", Age = 27 },
                new Employee { ID = 4, FirstName = "Alice", LastName = "Johnson", Country = "UK", Age = 28 },
                new Employee { ID = 5, FirstName = "Charlie", LastName = "Brown", Country = "USA", Age = 29 },
                new Employee { ID = 6, FirstName = "David", LastName = "Lee", Country = "China", Age = 30 },
                new Employee { ID = 7, FirstName = "Emily", LastName = "Wang", Country = "China", Age = 31 },
                new Employee { ID = 8, FirstName = "Frank", LastName = "Zhang", Country = "China", Age = 32 },
                new Employee { ID = 9, FirstName = "Grace", LastName = "Kim", Country = "Korea", Age = 33 },
                new Employee { ID = 10, FirstName = "Henry", LastName = "Park", Country = "Korea", Age = 34 },
                new Employee { ID = 11, FirstName = "Isabella", LastName = "Garcia", Country = "Mexico", Age = 35 },
                new Employee { ID = 12, FirstName = "Jack", LastName = "Hernandez", Country = "Mexico", Age = 36 },
                new Employee { ID = 13, FirstName = "Kaitlyn", LastName = "Nguyen", Country = "Vietnam", Age = 37 },
                new Employee { ID = 14, FirstName = "Liam", LastName = "Tran", Country = "Vietnam", Age = 38 },
                new Employee { ID = 15, FirstName = "Mia", LastName = "Chen", Country = "Taiwan", Age = 39 },
                new Employee { ID = 16, FirstName = "Noah", LastName = "Lin", Country = "Taiwan", Age = 40 },
                new Employee { ID = 17, FirstName = "Olivia", LastName = "Wu", Country = "Taiwan", Age = 41 },
                new Employee { ID = 18, FirstName = "Peter", LastName = "Choi", Country = "Korea", Age = 42 },
                new Employee { ID = 19, FirstName = "Quinn", LastName = "Kim", Country = "Korea", Age = 43 },
                new Employee { ID = 20, FirstName = "Rachel", LastName = "Lee", Country = "China", Age = 44 }
            };
        }
    }
}
